import "Turbine";
import "Turbine.UI";
import "Turbine.UI.Lotro";

import "CubePlugins.OpaqueQuestTracker.HelperFunctions"
import "CubePlugins.OpaqueQuestTracker.VindarPatch"

pluginName = plugin:GetName();
pluginVersion = plugin:GetVersion();
pluginDescription = string.format("'%s' v%s, by Cube", pluginName, pluginVersion);

settingsDataScope = Turbine.DataScope.Character;
settingsFilename = "OpaqueQuestTracker_Settings";

screenWidth, screenHeight = Turbine.UI.Display:GetSize();

-- Language stuff!
clientLanguage = Turbine.Engine:GetLanguage();

EN = Turbine.Language.English;
DE = Turbine.Language.German;
FR = Turbine.Language.French;


_LANG = {
    ["STATUS"] = {
        ["LOADED"] = {
            [EN] = "Loaded " .. pluginDescription;
            [DE] = "Geladen " .. pluginDescription;
        };
        ["UNLOADED"] = {
            [EN] = string.format("'%s' unloaded", pluginName);
        };
    };
    ["OPTIONS"] = {
        ["OPACITY"] = {
            [EN] = "Opacity: %d%%";
            [DE] = "Opazität: %d%%";
            [FR] = "Opacité: %d%%";
        };
        ["LOCKED"] = {
            [EN] = "Lock the window in place";
        };
    };
};

function GetString(text)
    -- use clientLanguage, it's always right

    -- If they passed in a non-existant thing, return an empty string
    if (text == nil) then return ""; end

    -- If the text is present in the language, return it
    if (text[clientLanguage] ~= nil) then return text[clientLanguage]; end

    -- Otherwise, fall back to English
    return text[EN];
end

-- this will be true if the number is formatted with a 
-- comma for the decimal place / radix point, false otherwise
isEuroFormat=(tonumber("1,000")==1);

-- create a function to automatcially convert in string format to number:
if (isEuroFormat) then
    function euroNormalize(value)
        return tonumber((string.gsub(value, "%.", ",")));
    end
else
    function euroNormalize(value)
        return tonumber((string.gsub(value, ",", ".")));
    end
end

-- end Language stuff!

DEFAULT_SETTINGS = {
    ["OPAQUE_WIN"] = {
        ["LEFT"] = 0.75;
        ["TOP"] = 0.35;
        ["WIDTH"] = 0.25;
        ["HEIGHT"] = 0.35;
        ["OPACITY"] = 0.5;
        ["LOCKED"] = false;
    };
};

SETTINGS = {};

function LoadSettings()
    local loadedSettings = PatchDataLoad(
        settingsDataScope,
        settingsFilename);

    -- did we load something good?
    if (type(loadedSettings) == 'table') then
        -- Yes, use what we loaded
        SETTINGS = loadedSettings;

        SETTINGS.OPAQUE_WIN.LEFT = euroNormalize(SETTINGS.OPAQUE_WIN.LEFT);
        SETTINGS.OPAQUE_WIN.TOP = euroNormalize(SETTINGS.OPAQUE_WIN.TOP);
        SETTINGS.OPAQUE_WIN.WIDTH = euroNormalize(SETTINGS.OPAQUE_WIN.WIDTH);
        SETTINGS.OPAQUE_WIN.HEIGHT = euroNormalize(SETTINGS.OPAQUE_WIN.HEIGHT);
        SETTINGS.OPAQUE_WIN.OPACITY = euroNormalize(SETTINGS.OPAQUE_WIN.OPACITY);
    else
        -- No, start with the default values, they're OK.
        SETTINGS = DEFAULT_SETTINGS;
    end
end

function SaveSettings()
    PatchDataSave(
        settingsDataScope,
        settingsFilename,
        SETTINGS);
end

function RegisterForUnload()
    Turbine.Plugin.Unload = function(sender, args)
        SaveSettings();

        Turbine.Shell.WriteLine(GetString(_LANG.STATUS.UNLOADED));
    end
end

function ChangeWindowOpacity(newOpacity)
    window:SetBackColor(Turbine.UI.Color(newOpacity, 0, 0, 0));
end

function CreateMainWindow()
    window = Turbine.UI.Window();
    window:SetPosition(SETTINGS.OPAQUE_WIN.LEFT * screenWidth, SETTINGS.OPAQUE_WIN.TOP * screenHeight);
    window:SetSize(SETTINGS.OPAQUE_WIN.WIDTH * screenWidth, SETTINGS.OPAQUE_WIN.HEIGHT * screenHeight);
    ChangeWindowOpacity(SETTINGS.OPAQUE_WIN.OPACITY);
    window:SetVisible(true);
    window:SetMouseVisible(not SETTINGS.OPAQUE_WIN.LOCKED);
    window:SetZOrder(-1);

    window.MouseDown = function(sender, args)
        window.mouseDown_MousePosition = { Turbine.UI.Display.GetMousePosition(); }
        window.mouseDown_WindowPosition = { window:GetPosition(); }
        window.mouseDown_WindowSize = { window:GetSize(); }
        window.isMouseDown = true;

        local borderWidth, borderHeight = 10, 10;
        local windowWidth, windowHeight = window:GetSize();
        local relativeMouseX, relativeMouseY = window:GetMousePosition();

        window.isMouseOverLeftBorder = relativeMouseX < borderWidth;
        window.isMouseOverTopBorder = relativeMouseY < borderHeight;
        window.isMouseOverRightBorder = relativeMouseX > (windowWidth - borderWidth);
        window.isMouseOverBottomBorder = relativeMouseY > (windowHeight - borderHeight);

        window.isMouseOverBorder = 
            window.isMouseOverLeftBorder or
            window.isMouseOverTopBorder or
            window.isMouseOverRightBorder or
            window.isMouseOverBottomBorder
    end

    window.MouseUp = function(sender, args)
        window.isMouseDown = false;
    end

    window.MouseMove = function(sender, args)
        if (window.isMouseDown) then
            local mouseDownX, mouseDownY = unpack(window.mouseDown_MousePosition);
            local mouseCurrentX, mouseCurrentY = Turbine.UI.Display.GetMousePosition();
            local windowLeft, windowTop = unpack(window.mouseDown_WindowPosition);

            local displayWidth, displayHeight = Turbine.UI.Display:GetSize();
            if (mouseCurrentX > displayWidth) then mouseCurrentX = displayWidth; end
            if (mouseCurrentY > displayHeight) then mouseCurrentY = displayHeight; end          

            if (window.isMouseOverBorder) then
                -- Resize the window!
                local mouseDown_WindowWidth, mouseDown_WindowHeight = unpack(window.mouseDown_WindowSize);

                -- how much will the width and height change?
                local deltaWidth = 0;
                local deltaHeight = 0;

                -- where will the window be?
                local left = windowLeft;
                local top = windowTop

                -- check for each of the borders:

                if (window.isMouseOverLeftBorder) then
                    -- make wider / narrower, adjust width to match:
                    -- left is bigger, right is smaller:
                    deltaWidth = mouseDownX - mouseCurrentX;
                    left = windowLeft - deltaWidth;
                end

                if (window.isMouseOverRightBorder) then
                    -- make wider / narrower, adjust width to match:
                    -- left is smaller, right is bigger:
                    deltaWidth = mouseCurrentX - mouseDownX;
                end

                if (window.isMouseOverTopBorder) then
                    -- make taller / shorter, adjust height to match:
                    -- up is bigger, down is smaller:
                    deltaHeight = mouseDownY - mouseCurrentY;
                    top = windowTop - deltaHeight;
                end

                if (window.isMouseOverBottomBorder) then
                    -- make taller / shorter, adjust height to match:
                    -- down is bigger, up is smaller:
                    deltaHeight = mouseCurrentY - mouseDownY;
                end

                -- Adjust the width and height based on which borders were moved:
                local width = mouseDown_WindowWidth + deltaWidth;
                local height = mouseDown_WindowHeight + deltaHeight;

                -- Don't let the window get too small:
                if (width < 200) then width = 200; end
                if (height < 200) then heigh = 200; end

                -- Update the window size (and if necessary position):
                window:SetSize(width, height);
                window:SetPosition(left, top);

            else
                -- Move the window!

                -- calculate how much the cursor has moved
                local deltaX = mouseCurrentX - mouseDownX;
                local deltaY = mouseCurrentY - mouseDownY;

                -- move the window the same distance that the mouse has moved:
                window:SetPosition(windowLeft + deltaX, windowTop + deltaY);

            end
            OnScreen(window);

            local screenWidth, screenHeight = Turbine.UI.Display:GetSize();
            SETTINGS.OPAQUE_WIN.LEFT = window:GetLeft() / screenWidth;
            SETTINGS.OPAQUE_WIN.TOP = window:GetTop() / screenHeight;
            SETTINGS.OPAQUE_WIN.WIDTH = window:GetWidth() / screenWidth;
            SETTINGS.OPAQUE_WIN.HEIGHT = window:GetHeight() / screenHeight;
        end
    end
end

function DrawOptionsControl()
    options = Turbine.UI.Control();
    plugin.GetOptionsPanel = function(self) return options; end

    options:SetBackColor(Turbine.UI.Color(0.1, 0.1, 0.1)); -- RGB, 0 = 0, 1 = 255.
    options:SetSize(250, 125);

    -- add a label for the scrollbar
    local opacityLabel = Turbine.UI.Label();
    opacityLabel:SetParent(options);
    opacityLabel:SetSize(200, 25);
    opacityLabel:SetText(string.format(GetString(_LANG.OPTIONS.OPACITY), SETTINGS.OPAQUE_WIN.OPACITY * 100));
    opacityLabel:SetPosition(10, 10);

    -- add a scrollbar to control opacity
    local opacityScrollBar = Turbine.UI.Lotro.ScrollBar();
    opacityScrollBar:SetParent(options);
    opacityScrollBar:SetSize(200, 10);
    opacityScrollBar:SetOrientation(Turbine.UI.Orientation.Horizontal);
    opacityScrollBar:SetPosition(10, 30);
    opacityScrollBar:SetValue(SETTINGS.OPAQUE_WIN.OPACITY * 100);
    opacityScrollBar.ValueChanged = function(sender, args)
        local value = opacityScrollBar:GetValue(); -- [0, 100]
        local scaledOpacity = value / 100;
        SETTINGS.OPAQUE_WIN.OPACITY = scaledOpacity;
        ChangeWindowOpacity(scaledOpacity); -- [0, 1]
        opacityLabel:SetText(string.format("Opacity: %d%%", value));
    end

    -- add a checkbox to "lock" the window
    local lockedCheckbox = Turbine.UI.Lotro.CheckBox();
    lockedCheckbox:SetParent(options);
    lockedCheckbox:SetPosition(10, 70);
    lockedCheckbox:SetSize(250, 25);
    lockedCheckbox:SetText(GetString(_LANG.OPTIONS.LOCKED));
    lockedCheckbox:SetChecked(SETTINGS.OPAQUE_WIN.LOCKED);
    lockedCheckbox.CheckedChanged = function(sender, args)
        local isLocked = lockedCheckbox:IsChecked();
        local mouseVisible = not isLocked;
        window:SetMouseVisible(mouseVisible);
        SETTINGS.OPAQUE_WIN.LOCKED = isLocked;
    end

    options.SizeChanged = function(sender, args)
        local width, height = options:GetSize();
        local margin = lockedCheckbox:GetLeft();
        lockedCheckbox:SetWidth(width - margin * 2);
    end
end

function Main()
    LoadSettings();
    RegisterForUnload();
    CreateMainWindow();
    DrawOptionsControl();

    Turbine.Shell.WriteLine(GetString(_LANG.STATUS.LOADED));
end

Main();
